@@ -74,15 +74,84 @@ static void wp_properties_help (GtkWindow * parent, char const * helpfile,
   }
 }
 
-static void gnome_wp_file_open_cancel (GtkWidget * widget, gpointer data) {
-  gtk_widget_hide (gtk_widget_get_toplevel (widget));
+static void gnome_wp_add_image (GnomeWPCapplet * capplet,
+				const gchar * filename) {
+  GnomeWPItem * item;
+  GtkTreePath * path;
+  GdkColor color1, color2;
+
+  item = g_hash_table_lookup (capplet->wphash, filename);
+  if (item != NULL) {
+    GtkTreePath * path;
+    
+    if (item->deleted) {
+      item->deleted = FALSE;
+      wp_props_load_wallpaper (item->filename, item, capplet);
+    }
+
+    path = gtk_tree_row_reference_get_path (item->rowref);
+    gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
+			      NULL, FALSE);
+    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
+				  path, NULL, TRUE, 0.5, 0.0);
+    gtk_tree_path_free (path);
+
+    return;
+  }
+
+  item = g_new0 (GnomeWPItem, 1);
+  
+  item->filename = g_strdup (filename);
+
+  item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
+
+  item->shade_type = gconf_client_get_string (capplet->client,
+					      WP_SHADING_KEY, NULL);
+  item->pri_color = gconf_client_get_string (capplet->client,
+					     WP_PCOLOR_KEY, NULL);
+  item->sec_color = gconf_client_get_string (capplet->client,
+					     WP_SCOLOR_KEY, NULL);
+
+  gdk_color_parse (item->pri_color, &color1);
+  gdk_color_parse (item->sec_color, &color2);
+     
+  item->pcolor = gdk_color_copy (&color1);
+  item->scolor = gdk_color_copy (&color2);
+
+  if (!strncmp (item->fileinfo->mime_type, "image/", strlen ("image/"))) {
+    if (item->name == NULL) {
+      item->name = g_strdup (item->fileinfo->name);
+    }
+    item->options = gconf_client_get_string (capplet->client,
+					     WP_OPTIONS_KEY,
+					     NULL);
+    if (!strcmp (item->options, "none")) {
+      item->options = g_strdup ("wallpaper");
+    }
+    gnome_wp_item_update_description (item);
+     
+    g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
+    wp_props_load_wallpaper (item->filename, item, capplet);
+
+    gconf_client_set_string (capplet->client, WP_FILE_KEY,
+			     item->filename, NULL);
+    gconf_client_set_string (capplet->client, WP_OPTIONS_KEY,
+			     item->options, NULL);
+
+    path = gtk_tree_row_reference_get_path (item->rowref);
+    gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
+			      NULL, FALSE);
+    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
+				  path, NULL, TRUE, 0.5, 0.0);
+    gtk_tree_path_free (path);
+  } else {
+    gnome_wp_item_free (item);
+  }
 }
 
-static void gnome_wp_file_open_get_files (GtkWidget * widget,
-					  GnomeWPCapplet * capplet) {
-  GtkWidget * filesel;
-  GdkColor color1, color2;
-  gchar ** files;
+static void gnome_wp_add_images (GnomeWPCapplet * capplet,
+				 GSList * images,
+				 gchar ** files) {
   GdkCursor * cursor;
   gint i;
 
@@ -91,92 +160,41 @@ static void gnome_wp_file_open_get_files (GtkWidget * widget,
   gdk_window_set_cursor (capplet->window->window, cursor);
   gdk_cursor_unref (cursor);
 
-  filesel = gtk_widget_get_toplevel (widget);
-
-  files = gtk_file_selection_get_selections (GTK_FILE_SELECTION (filesel));
-
-  for (i = 0; files && files[i]; i++) {
-    GnomeWPItem * item;
-
-    item = g_hash_table_lookup (capplet->wphash, files[i]);
-    if (item != NULL) {
-      GtkTreePath * path;
-
-      path = gtk_tree_row_reference_get_path (item->rowref);
-      gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				NULL, FALSE);
-      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				    path, NULL, TRUE, 0.5, 0.0);
-      gtk_tree_path_free (path);
-
-      continue;
+  if (images != NULL) {
+    for (; images != NULL; images = images->next) {
+      gnome_wp_add_image (capplet, images->data);
+    }
+  } else {
+    for (i = 0; files && files[i]; i++) {
+      gnome_wp_add_image (capplet, files[i]);
     }
-
-   item = g_new0 (GnomeWPItem, 1);
-
-   item->filename = g_strdup (files[i]);
-
-   item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-
-   item->shade_type = gconf_client_get_string (capplet->client,
-					       WP_SHADING_KEY, NULL);
-   item->pri_color = gconf_client_get_string (capplet->client,
-					      WP_PCOLOR_KEY, NULL);
-   item->sec_color = gconf_client_get_string (capplet->client,
-					      WP_SCOLOR_KEY, NULL);
-
-   gdk_color_parse (item->pri_color, &color1);
-   gdk_color_parse (item->sec_color, &color2);
-     
-   item->pcolor = gdk_color_copy (&color1);
-   item->scolor = gdk_color_copy (&color2);
-
-   if (!strncmp (item->fileinfo->mime_type, "image/", strlen ("image/"))) {
-     if (item->name == NULL) {
-       item->name = g_strdup (item->fileinfo->name);
-     }
-     item->options = gconf_client_get_string (capplet->client,
-					      WP_OPTIONS_KEY,
-					      NULL);
-
-     item->description = g_strdup_printf ("<b>%s</b>\n%s (%LuK)",
-					  item->name,
-					  gnome_vfs_mime_get_description (item->fileinfo->mime_type),
-					  item->fileinfo->size / 1024);
-     
-     g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
-     wp_props_load_wallpaper (item->filename, item, capplet);
-   } else {
-     gnome_wp_item_free (item);
-   }
   }
 
-  g_strfreev (files);
-  gtk_widget_hide (filesel);
-
   gdk_window_set_cursor (capplet->window->window, NULL);
 }
 
 static void gnome_wp_file_open_dialog (GtkWidget * widget,
 				       GnomeWPCapplet * capplet) {
-  static GtkWidget * filesel = NULL;
+#if GTK_CHECK_VERSION (2, 3, 0)
+  GSList * files;
+#else
+  gchar ** files;
+#endif
 
-  if (filesel != NULL) {
-    gtk_widget_show (filesel);
-    return;
+  switch (gtk_dialog_run (GTK_DIALOG (capplet->filesel))) {
+  case GTK_RESPONSE_OK:
+#if GTK_CHECK_VERSION (2, 3, 0)
+    files = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (capplet->filesel));
+    gnome_wp_add_images (capplet, files, NULL);
+#else
+    files = gtk_file_selection_get_selections (GTK_FILE_SELECTION (capplet->filesel));
+    gnome_wp_add_images (capplet, NULL, files);
+#endif
+  case GTK_RESPONSE_CANCEL:
+  default:
+    gtk_widget_hide (capplet->filesel);
+    break;
   }
-
-  filesel = gtk_file_selection_new (_("Add Wallpapers"));
-  gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (filesel), TRUE);
-
-  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
-		    "clicked",
-		    G_CALLBACK (gnome_wp_file_open_get_files), capplet);
-  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
-		    "clicked",
-		    G_CALLBACK (gnome_wp_file_open_cancel), capplet);
-
-  gtk_widget_show (filesel);
 }
 
 static void bg_add_multiple_files (GnomeVFSURI * uri,
@@ -211,10 +229,7 @@ static void bg_add_multiple_files (GnomeVFSURI * uri,
   item->pcolor = gdk_color_copy (&color1);
   item->scolor = gdk_color_copy (&color2);
 
-  item->description = g_strdup_printf ("<b>%s</b>\n%s (%LuK)",
-				       item->name,
-				       gnome_vfs_mime_get_description (item->fileinfo->mime_type),
-				       item->fileinfo->size / 1024);
+  gnome_wp_item_update_description (item);
   
   g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
   wp_props_load_wallpaper (item->filename, item, capplet);
@@ -266,14 +281,14 @@ static void bg_properties_dragged_image (GtkWidget * widget,
 	  item->pcolor = gdk_color_copy (&color1);
 	  item->scolor = gdk_color_copy (&color2);
 
-	  item->description = g_strdup_printf ("<b>%s</b>\n%s (%LuK)",
-					       item->name,
-					       gnome_vfs_mime_get_description (item->fileinfo->mime_type),
-					       item->fileinfo->size / 1024);
+	  gnome_wp_item_update_description (item);
 
 	  g_hash_table_insert (capplet->wphash, g_strdup (item->filename),
 			       item);
 	  wp_props_load_wallpaper (item->filename, item, capplet);
+	} else if (item->deleted) {
+	  item->deleted = FALSE;
+	  wp_props_load_wallpaper (item->filename, item, capplet);
 	}
 	gconf_client_set_string (capplet->client, WP_FILE_KEY,
 				 item->filename, NULL);
@@ -317,6 +332,7 @@ static void wp_props_load_wallpaper (gchar * key,
   gtk_list_store_append (GTK_LIST_STORE (capplet->model), &iter);
 
   pixbuf = gnome_wp_item_get_thumbnail (item, capplet->thumbs);
+  gnome_wp_item_update_description (item);
 
   if (pixbuf != NULL) {
     gtk_list_store_set (GTK_LIST_STORE (capplet->model), &iter,
@@ -389,6 +405,7 @@ static gboolean gnome_wp_props_wp_set (GnomeWPCapplet * capplet) {
   GtkTreeModel * model;
   GtkTreeSelection * selection;
   GnomeWPItem * item;
+  GConfChangeSet * cs;
   gchar * wpfile;
   GdkPixbuf * pixbuf;
 
@@ -398,29 +415,29 @@ static gboolean gnome_wp_props_wp_set (GnomeWPCapplet * capplet) {
 
     item = g_hash_table_lookup (capplet->wphash, wpfile);
 
+    cs = gconf_change_set_new ();
+
     if (!strcmp (item->filename, "(none)")) {
-      gconf_client_set_string (capplet->client, WP_OPTIONS_KEY,
-			       "none", NULL);
+      gconf_change_set_set_string (cs, WP_OPTIONS_KEY, "none");
       gtk_widget_set_sensitive (capplet->wp_opts, FALSE);
       gtk_widget_set_sensitive (capplet->rm_button, FALSE);
     } else {
       gtk_widget_set_sensitive (capplet->wp_opts, TRUE);
       gtk_widget_set_sensitive (capplet->rm_button, TRUE);
-      gconf_client_set_string (capplet->client, WP_FILE_KEY,
-			       item->filename, NULL);
-      gconf_client_set_string (capplet->client, WP_OPTIONS_KEY,
-			       item->options, NULL);
+      gconf_change_set_set_string (cs, WP_FILE_KEY, item->filename);
+      gconf_change_set_set_string (cs, WP_OPTIONS_KEY, item->options);
       gnome_wp_option_menu_set (capplet, item->options, FALSE);
     }
 
-    gconf_client_set_string (capplet->client, WP_SHADING_KEY,
-			     item->shade_type, NULL);
+    gconf_change_set_set_string (cs, WP_SHADING_KEY, item->shade_type);
     gnome_wp_option_menu_set (capplet, item->shade_type, TRUE);
 
-    gconf_client_set_string (capplet->client, WP_PCOLOR_KEY,
-			     item->pri_color, NULL);
-    gconf_client_set_string (capplet->client, WP_SCOLOR_KEY,
-			     item->sec_color, NULL);
+    gconf_change_set_set_string (cs, WP_PCOLOR_KEY, item->pri_color);
+    gconf_change_set_set_string (cs, WP_SCOLOR_KEY, item->sec_color);
+
+    gconf_client_commit_change_set (capplet->client, cs, TRUE, NULL);
+
+    gconf_change_set_unref (cs);
 
     gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (capplet->pc_picker),
 				item->pcolor->red,
@@ -496,7 +513,7 @@ static void wallpaper_properties_clicked (GtkWidget * dialog,
   switch (response_id) {
   case GTK_RESPONSE_HELP:
     wp_properties_help (GTK_WINDOW (dialog),
-			"wgoscustdesk.xml", "goscustdesk-7");
+			"user-guide.xml", "goscustdesk-7");
     break;
   case GTK_RESPONSE_CLOSE: {
     gtk_widget_destroy (dialog);
@@ -509,6 +526,7 @@ static void wallpaper_properties_clicked (GtkWidget * dialog,
 static void gnome_wp_scale_type_changed (GtkMenuShell * shell,
 					 GnomeWPCapplet * capplet) {
   GnomeWPItem * item = NULL;
+  GdkPixbuf * pixbuf;
   GtkTreeIter iter;
   GtkTreeModel * model;
   GtkTreeSelection * selection;
@@ -542,6 +560,11 @@ static void gnome_wp_scale_type_changed (GtkMenuShell * shell,
   default:
     break;
   }
+  pixbuf = gnome_wp_item_get_thumbnail (item, capplet->thumbs);
+  gtk_list_store_set (GTK_LIST_STORE (capplet->model), &iter,
+		      0, pixbuf,
+		      -1);
+  g_object_unref (pixbuf);
   gconf_client_set_string (capplet->client, WP_OPTIONS_KEY,
 			   item->options, NULL);
 }
@@ -672,6 +695,7 @@ static void gnome_wp_remove_wallpaper (GtkWidget * widget,
 				       GnomeWPCapplet * capplet) {
   GtkTreeIter iter;
   GtkTreeModel * model;
+  GtkTreePath * first;
   GtkTreeSelection * selection;
   gchar * wpfile;
 
@@ -690,14 +714,22 @@ static void gnome_wp_remove_wallpaper (GtkWidget * widget,
 
     gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
   }
+  first = gtk_tree_path_new_first ();
+  gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview),
+			    first, NULL, FALSE);
+  gtk_tree_path_free (first);
 }
 
 static gboolean gnome_wp_load_stuffs (void * data) {
   GnomeWPCapplet * capplet = (GnomeWPCapplet *) data;
-  gchar * imagepath;
+  gchar * imagepath, * style;
   GnomeWPItem * item;
   GtkTreePath * path;
 
+  style = gconf_client_get_string (capplet->client,
+				   WP_OPTIONS_KEY,
+				   NULL);
+
   gnome_wp_xml_load_list (capplet);
   g_hash_table_foreach (capplet->wphash, (GHFunc) wp_props_load_wallpaper,
 			capplet);
@@ -707,8 +739,9 @@ static gboolean gnome_wp_load_stuffs (void * data) {
   imagepath = gconf_client_get_string (capplet->client,
 				       WP_FILE_KEY,
 				       NULL);
+
   item = g_hash_table_lookup (capplet->wphash, imagepath);
-  if (item != NULL) {
+  if (item != NULL && strcmp (style, "none") != 0) {
     if (item->deleted == TRUE) {
       item->deleted = FALSE;
       wp_props_load_wallpaper (item->filename, item, capplet);
@@ -732,7 +765,7 @@ static gboolean gnome_wp_load_stuffs (void * data) {
 				item->scolor->red,
 				item->scolor->green,
 				item->scolor->blue, 65535);
-  } else {
+  } else if (strcmp (style, "none") != 0) {
     GdkColor color1, color2;
 
     item = g_new0 (GnomeWPItem, 1);
@@ -760,10 +793,7 @@ static gboolean gnome_wp_load_stuffs (void * data) {
 					       NULL);
       item->deleted = FALSE;
 
-      item->description = g_strdup_printf ("<b>%s</b>\n%s (%LuK)",
-					   item->name,
-					   gnome_vfs_mime_get_description (item->fileinfo->mime_type),
-					   item->fileinfo->size / 1024);
+      gnome_wp_item_update_description (item);
 
       g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
       wp_props_load_wallpaper (item->filename, item, capplet);
@@ -790,7 +820,6 @@ static gboolean gnome_wp_load_stuffs (void * data) {
       gnome_wp_item_free (item);
     }
   }
-  g_free (imagepath);
 
   item = g_hash_table_lookup (capplet->wphash, "(none)");
   if (item == NULL) {
@@ -820,7 +849,7 @@ static gboolean gnome_wp_load_stuffs (void * data) {
 					     WP_OPTIONS_KEY,
 					     NULL);
 
-    item->description = g_strdup_printf ("<b>%s</b>", item->name);
+    gnome_wp_item_update_description (item);
 
     g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
     wp_props_load_wallpaper (item->filename, item, capplet);
@@ -829,7 +858,18 @@ static gboolean gnome_wp_load_stuffs (void * data) {
       item->deleted = FALSE;
       wp_props_load_wallpaper (item->filename, item, capplet);
     }
+
+    if (!strcmp (style, "none")) {
+      path = gtk_tree_row_reference_get_path (item->rowref);
+      gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
+				NULL, FALSE);
+      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
+				    path, NULL, TRUE, 0.5, 0.0);
+      gtk_tree_path_free (path);
+    }
   }
+  g_free (imagepath);
+  g_free (style);
 
   return FALSE;
 }
@@ -848,7 +888,7 @@ static gint gnome_wp_list_sort (GtkTreeModel * model,
   } else if (!strcmp (bar, "(none)")) {
     return 1;
   } else {
-    return strcmp (desca, descb);
+    return g_utf8_collate (desca, descb);
   }
 }
 
@@ -903,11 +943,7 @@ static void gnome_wp_file_changed (GConfClient * client, guint id,
 						   WP_OPTIONS_KEY,
 						   NULL);
       
-	  item->description = g_strdup_printf ("<b>%s</b>\n%s (%LuK)",
-					       item->name,
-					       gnome_vfs_mime_get_description (item->fileinfo->mime_type),
-					       item->fileinfo->size / 1024);
-      
+	  gnome_wp_item_update_description (item);
 	  g_hash_table_insert (capplet->wphash,
 			       g_strdup (item->filename), item);
 	  wp_props_load_wallpaper (item->filename, item, capplet);
@@ -1018,13 +1054,17 @@ static void gnome_wp_delay_changed (GConfClient * client, guint id,
 
 static void gnome_wp_icon_theme_changed (GnomeIconTheme * theme,
 					 GnomeWPCapplet * capplet) {
+  GdkPixbuf * pixbuf;
   gchar * icofile;
 
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
 					  "background-capplet",
 					  48, NULL, NULL);
   if (icofile != NULL) {
+    pixbuf = gdk_pixbuf_new_from_file (icofile, NULL);
+    gtk_window_set_icon (GTK_WINDOW (capplet->window), NULL);
     gtk_window_set_default_icon_from_file (icofile, NULL);
+    g_object_unref (pixbuf);
   }
   g_free (icofile);
 
@@ -1077,19 +1117,49 @@ static void gnome_wp_icon_theme_changed (GnomeIconTheme * theme,
   g_free (icofile);
 }
 
+static GladeXML * gnome_wp_create_dialog (void) {
+  GladeXML * new;
+  gchar * gladefile;
+
+  gladefile = g_build_filename (GNOMECC_DATA_DIR,
+				"interfaces",
+				"gnome-background-properties.glade",
+				NULL);
+
+  if (!g_file_test (gladefile, G_FILE_TEST_EXISTS)) {
+    gladefile = g_build_filename (g_get_current_dir (),
+				  "gnome-background-properties.glade",
+				  NULL);
+  }
+  new = glade_xml_new (gladefile, NULL, NULL);
+  g_free (gladefile);
+
+  return new;
+}
+
 static void wallpaper_properties_init (void) {
   GnomeWPCapplet * capplet;
-  GtkWidget * label, * button;
-  GtkWidget * vbox, * hbox, * bbox;
-  GtkWidget * swin, * clabel;
-  GtkWidget * menu;
+  GladeXML * dialog;
+  GtkWidget * menu, * label;
   GtkWidget * mbox, * mitem;
+  GtkWidget * add_button;
   GtkCellRenderer * renderer;
   GtkTreeViewColumn * column;
   GtkTreeSelection * selection;
+  GdkPixbuf * pixbuf;
   GdkCursor * cursor;
   gchar * icofile;
 
+  gtk_rc_parse_string ("style \"wp-tree-defaults\" {\n"
+		       "  GtkTreeView::horizontal-separator = 6\n"
+		       "} widget_class \"*TreeView*\""
+		       " style \"wp-tree-defaults\"\n\n"
+		       "style \"wp-dialog-defaults\" {\n"
+		       "  GtkDialog::action-area-border = 0\n"
+		       "  GtkDialog::content-area-border = 0\n"
+		       "} widget_class \"*GtkDialog*\""
+		       " style \"wp-dialog-defaults\"");
+
   capplet = g_new0 (GnomeWPCapplet, 1);
 
   if (capplet->client == NULL) {
@@ -1141,23 +1211,23 @@ static void wallpaper_properties_init (void) {
   g_signal_connect (G_OBJECT (capplet->theme), "changed",
 		    G_CALLBACK (gnome_wp_icon_theme_changed), capplet);
 
-  capplet->window = gtk_dialog_new ();
-  gtk_window_set_title (GTK_WINDOW (capplet->window),
-			_("Desktop Wallpaper Preferences"));
-  gtk_dialog_set_has_separator (GTK_DIALOG (capplet->window), FALSE);
-  gtk_window_set_default_size (GTK_WINDOW (capplet->window), 360, 418);
+  dialog = gnome_wp_create_dialog ();
+  capplet->window = glade_xml_get_widget (dialog,"gnome_wp_properties");
 
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
 					  "background-capplet",
 					  48, NULL, NULL);
   if (icofile != NULL) {
+    pixbuf = gdk_pixbuf_new_from_file (icofile, NULL);
     gtk_window_set_default_icon_from_file (icofile, NULL);
+    gtk_window_set_icon (GTK_WINDOW (capplet->window), pixbuf);
+    g_object_unref (pixbuf);
   }
   g_free (icofile);
 
   gtk_widget_realize (capplet->window);
 
-
+  gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (capplet->window)->action_area), 12);
 
   /* Drag and Drop Support */
   gtk_drag_dest_unset (capplet->window);
@@ -1167,60 +1237,7 @@ static void wallpaper_properties_init (void) {
   g_signal_connect (G_OBJECT (capplet->window), "drag_data_received",
 		    G_CALLBACK (bg_properties_dragged_image), capplet);
 
-  /* Dialog Buttons */
-  /* We are hiding the help button for now, since we have no docs */
-  /*
-  label = gtk_button_new_from_stock (GTK_STOCK_HELP);
-  gtk_dialog_add_action_widget (GTK_DIALOG (capplet->window), label,
-				GTK_RESPONSE_HELP);
-  gtk_widget_show (label);
-  */
-
-  label = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
-  gtk_dialog_add_action_widget (GTK_DIALOG (capplet->window), label,
-				GTK_RESPONSE_CLOSE);
-  gtk_widget_show (label);
-
-  /* Main Contents */
-  vbox = gtk_vbox_new (FALSE, 6);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (capplet->window)->vbox), vbox,
-		      TRUE, TRUE, 0);
-  gtk_widget_show (vbox);
-
-  clabel = gtk_label_new_with_mnemonic (_("<b>Desktop _Wallpaper</b>"));
-  gtk_misc_set_alignment (GTK_MISC (clabel), 0.0, 0.5);
-  gtk_label_set_use_markup (GTK_LABEL (clabel), TRUE);
-  gtk_box_pack_start (GTK_BOX (vbox), clabel, FALSE, FALSE, 0);
-  gtk_widget_show (clabel);
-
-  /* Treeview stuff goes in here */
-  hbox = gtk_hbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-  gtk_widget_show (hbox);
-
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  swin = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin),
-				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin),
-				       GTK_SHADOW_IN);
-  gtk_box_pack_start (GTK_BOX (hbox), swin, TRUE, TRUE, 0);
-
-  capplet->treeview = gtk_tree_view_new ();
-  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (capplet->treeview), FALSE);
-  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (capplet->treeview), FALSE);
-  gtk_container_add (GTK_CONTAINER (swin), capplet->treeview);
-  gtk_widget_show (capplet->treeview);
-
-  gtk_label_set_mnemonic_widget (GTK_LABEL (clabel), capplet->treeview);
+  capplet->treeview = glade_xml_get_widget (dialog,"wp_tree");
 
   capplet->model = GTK_TREE_MODEL (gtk_list_store_new (3, GDK_TYPE_PIXBUF,
 						       G_TYPE_STRING,
@@ -1251,37 +1268,7 @@ static void wallpaper_properties_init (void) {
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (capplet->model),
 					2, GTK_SORT_ASCENDING);
 
-  gtk_rc_parse_string ("style \"wp-tree-defaults\" {\n"
-		       "  GtkTreeView::horizontal-separator = 6\n"
-		       "  GtkTreeView::vertical-separator = 6\n"
-		       "} widget_class \"*TreeView*\""
-		       " style \"wp-tree-defaults\"");
-
-  /* Need to add sorting stuff and whatnot */
-  gtk_widget_show (swin);
-
-  /* The Box for Fill Style and Add/Remove buttons */
-  hbox = gtk_hbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
-
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  label = gtk_label_new_with_mnemonic (_("_Style:"));
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  capplet->wp_opts = gtk_option_menu_new ();
-  gtk_box_pack_start (GTK_BOX (hbox), capplet->wp_opts, FALSE, FALSE, 0);
-  gtk_widget_show (capplet->wp_opts);
-
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label), capplet->wp_opts);
+  capplet->wp_opts = glade_xml_get_widget (dialog,"style_menu");
 
   menu = gtk_menu_new ();
   mitem = gtk_menu_item_new ();
@@ -1374,83 +1361,15 @@ static void wallpaper_properties_init (void) {
   g_signal_connect (G_OBJECT (menu), "deactivate",
 		    G_CALLBACK (gnome_wp_scale_type_changed), capplet);
 
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  /* Create the Remove button first, since it's ordered last */
-  capplet->rm_button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
-  gtk_box_pack_end (GTK_BOX (hbox), capplet->rm_button, FALSE, FALSE, 0);
-  gtk_widget_show (capplet->rm_button);
+  add_button = glade_xml_get_widget (dialog,"add_button");
+  capplet->rm_button = glade_xml_get_widget (dialog,"rem_button");
 
+  g_signal_connect (G_OBJECT (add_button), "clicked",
+		    G_CALLBACK (gnome_wp_file_open_dialog), capplet);
   g_signal_connect (G_OBJECT (capplet->rm_button), "clicked",
 		    G_CALLBACK (gnome_wp_remove_wallpaper), capplet);
 
-  /* Now do the Add Wallpaper button */
-  button = gtk_button_new ();
-  gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-
-  bbox = gtk_hbox_new (FALSE, 6);
-  gtk_container_add (GTK_CONTAINER (button), bbox);
-  label = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
-  gtk_box_pack_start (GTK_BOX (bbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  label = gtk_label_new_with_mnemonic (_("_Add Wallpaper"));
-  gtk_box_pack_start (GTK_BOX (bbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  gtk_widget_show (bbox);
-
-  gtk_widget_show (button);
-
-  g_signal_connect (G_OBJECT (button), "clicked",
-		    G_CALLBACK (gnome_wp_file_open_dialog), capplet);
-
-  /* Silly Random Option */
-  /*
-  hbox = gtk_hbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
-
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  */
-
-  /* Stupid Useless Label as a Spacer Hack */
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  /* The Wallpaper Background Colors Section */
-  clabel = gtk_label_new_with_mnemonic (_("<b>_Desktop Colors</b> "));
-  gtk_misc_set_alignment (GTK_MISC (clabel), 0.0, 0.5);
-  gtk_label_set_use_markup (GTK_LABEL (clabel), TRUE);
-  gtk_box_pack_start (GTK_BOX (vbox), clabel, FALSE, FALSE, 0);
-  gtk_widget_show (clabel);
-
-  hbox = gtk_hbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
-
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  capplet->color_opt = gtk_option_menu_new ();
-  gtk_widget_show (capplet->color_opt);
-  gtk_box_pack_start(GTK_BOX (hbox), capplet->color_opt, FALSE, FALSE, 0);
+  capplet->color_opt = glade_xml_get_widget (dialog,"color_menu");
 
   menu = gtk_menu_new ();
   mitem = gtk_menu_item_new ();
@@ -1505,16 +1424,11 @@ static void wallpaper_properties_init (void) {
   g_signal_connect (G_OBJECT (menu), "deactivate",
 		    G_CALLBACK (gnome_wp_shade_type_changed), capplet);
 
-  gtk_label_set_mnemonic_widget (GTK_LABEL (clabel), capplet->color_opt);
-
-  capplet->pc_picker = gnome_color_picker_new ();
-  gtk_widget_show (capplet->pc_picker);
-  gtk_box_pack_start (GTK_BOX (hbox), capplet->pc_picker, FALSE, FALSE, 0);
+  capplet->pc_picker = glade_xml_get_widget (dialog,"pcpicker");
   g_signal_connect (G_OBJECT (capplet->pc_picker), "color_set",
 		    G_CALLBACK (gnome_wp_pcolor_changed), capplet);
 
-  capplet->sc_picker = gnome_color_picker_new ();
-  gtk_box_pack_start (GTK_BOX (hbox), capplet->sc_picker, FALSE, FALSE, 0);
+  capplet->sc_picker = glade_xml_get_widget (dialog,"scpicker");
   g_signal_connect (G_OBJECT (capplet->sc_picker), "color_set",
 		    G_CALLBACK (gnome_wp_scolor_changed), capplet);
   
@@ -1533,6 +1447,24 @@ static void wallpaper_properties_init (void) {
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (capplet->treeview));
   g_signal_connect (G_OBJECT (selection), "changed",
 		    G_CALLBACK (gnome_wp_props_wp_selected), capplet);
+
+  /* Create the file chooser dialog stuff here */
+#if GTK_CHECK_VERSION (2, 3, 0)
+  capplet->filesel = gtk_file_chooser_dialog_new (_("Add Wallpapers"),
+						  GTK_WINDOW (capplet->window),
+						  GTK_FILE_CHOOSER_ACTION_OPEN,
+						  GTK_STOCK_CANCEL,
+						  GTK_RESPONSE_CANCEL,
+						  GTK_STOCK_OPEN,
+						  GTK_RESPONSE_OK,
+						  NULL);
+  gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (capplet->filesel),
+					TRUE);
+#else
+  capplet->filesel = gtk_file_selection_new (_("Add Wallpapers"));
+  gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (capplet->filesel),
+					  TRUE);
+#endif
 }
 
 gint main (gint argc, gchar *argv[]) {
