@@ -37,7 +37,7 @@
 
 #include "libuuid/uuid.h"
 
-#undef WID(x)
+#undef WID
 #define WID(x) (glade_xml_get_widget (dialog->p->dialog_xml, x))
 
 enum {
@@ -262,7 +262,7 @@ mime_edit_dialog_class_init (MimeEditDialogClass *class)
 		 g_param_spec_pointer ("mime-type-info",
 				       _("MIME type information"),
 				       _("Structure with data on the MIME type"),
-				       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+				       G_PARAM_READWRITE));
 
 	g_object_class_install_property
 		(object_class, PROP_IS_ADD,
@@ -270,7 +270,7 @@ mime_edit_dialog_class_init (MimeEditDialogClass *class)
 				       _("Is add dialog"),
 				       _("True if this dialog is for adding a MIME type"),
 				       FALSE,
-				       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+				       G_PARAM_READWRITE));
 
 	dialog_signals[DONE] =
 		g_signal_new ("done",
@@ -406,11 +406,9 @@ mime_add_dialog_new (GtkTreeModel *model, GtkWindow *parent,
 		     char const *file_name) 
 {
 	GObject *dialog = g_object_new (mime_edit_dialog_get_type (),
-		"model", model,	/* must be before is-add */
+		"model", model,	"is-add", TRUE, /* model must be before is-add */
 		NULL);
-	g_object_set (dialog,
-		      "is-add", TRUE,
-		      NULL);
+
 	if (parent != NULL)
 		gtk_window_set_transient_for (
 			GTK_WINDOW (MIME_EDIT_DIALOG (dialog)->p->dialog_win),
@@ -716,7 +714,7 @@ mime_edit_dialog_get_app (GladeXML *glade, char const *mime_type,
 
 			res->name = base_cmd;
 			if (res->name != NULL && *res->name) {
-				/* Can we use the app name as the id ?
+				/* Can we use the app name as the id?
 				 * We know that there are no apps with the same
 				 * command, so if the id is taken we are screwed
 				 */
@@ -856,7 +854,7 @@ validate_data (MimeEditDialog *dialog)
 				GTK_WINDOW (dialog->p->dialog_win),
 				GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_OK_CANCEL,
-				_("A MIME type with that name already exists, overwrite ?."));
+				_("A MIME type with that name already exists, overwrite?"));
 		}
 	} else {
 		err_dialog = gtk_message_dialog_new (
