@@ -18,6 +18,8 @@
 
 #define EGG_CELL_RENDERER_TEXT_PATH "egg-cell-renderer-text"
 
+#define TOOLTIP_TEXT _("<New accelerator...>")
+
 static void             egg_cell_renderer_keys_finalize      (GObject             *object);
 static void             egg_cell_renderer_keys_init          (EggCellRendererKeys *cell_keys);
 static void             egg_cell_renderer_keys_class_init    (EggCellRendererKeysClass *cell_keys_class);
@@ -343,7 +345,7 @@ is_modifier (guint keycode)
   return retval;
 }
 
-void
+static void
 egg_cell_renderer_keys_get_size (GtkCellRenderer *cell,
 				 GtkWidget       *widget,
 				 GdkRectangle    *cell_area,
@@ -351,12 +353,13 @@ egg_cell_renderer_keys_get_size (GtkCellRenderer *cell,
 				 gint            *y_offset,
 				 gint            *width,
 				 gint            *height)
+
 {
   EggCellRendererKeys *keys = (EggCellRendererKeys *) cell;
   GtkRequisition requisition;
 
   if (keys->sizing_label == NULL)
-    keys->sizing_label = gtk_label_new (_("Type a new accelerator, or press Backspace to clear"));
+    keys->sizing_label = gtk_label_new (TOOLTIP_TEXT);
 
   gtk_widget_size_request (keys->sizing_label, &requisition);
   (* GTK_CELL_RENDERER_CLASS (parent_class)->get_size) (cell, widget, cell_area, x_offset, y_offset, width, height);
@@ -421,6 +424,14 @@ grab_key_callback (GtkWidget    *widget,
                                         EGG_VIRTUAL_SCROLL_LOCK_MASK,
                                         &ignored_modifiers);
   
+  /* http://bugzilla.gnome.org/show_bug.cgi?id=139605
+   * mouse keys should effect keybindings */
+  ignored_modifiers |=	GDK_BUTTON1_MASK |
+			GDK_BUTTON2_MASK |
+			GDK_BUTTON3_MASK |
+			GDK_BUTTON4_MASK |
+			GDK_BUTTON5_MASK;
+
   /* filter consumed/ignored modifiers */
 
   if (keys->accel_mode == EGG_CELL_RENDERER_KEYS_MODE_GTK)
@@ -594,12 +605,8 @@ egg_cell_renderer_keys_start_editing (GtkCellRenderer      *cell,
   gtk_widget_modify_fg (label, GTK_STATE_NORMAL,
                         &widget->style->fg[GTK_STATE_SELECTED]);
   
-  if (keys->accel_key != 0)
-    gtk_label_set_text (GTK_LABEL (label),
-			_("Type a new accelerator, or press Backspace to clear"));
-  else
-    gtk_label_set_text (GTK_LABEL (label),
-                        _("Type a new accelerator"));
+  gtk_label_set_text (GTK_LABEL (label),
+		  TOOLTIP_TEXT);
 
   gtk_container_add (GTK_CONTAINER (eventbox), label);
   
@@ -620,7 +627,7 @@ void
 egg_cell_renderer_keys_set_accelerator (EggCellRendererKeys *keys,
                                         guint                keyval,
 					guint		     keycode,
-                                        GdkModifierType      mask)
+					EggVirtualModifierType  mask)
 {
   char *text;
   gboolean changed;
@@ -668,9 +675,9 @@ egg_cell_renderer_keys_set_accelerator (EggCellRendererKeys *keys,
 }
 
 void
-egg_cell_renderer_keys_get_accelerator (EggCellRendererKeys *keys,
-                                        guint               *keyval,
-                                        GdkModifierType     *mask)
+egg_cell_renderer_keys_get_accelerator (EggCellRendererKeys     *keys,
+                                        guint                   *keyval,
+                                        EggVirtualModifierType  *mask)
 {
   g_return_if_fail (EGG_IS_CELL_RENDERER_KEYS (keys));
 
