@@ -81,7 +81,7 @@ enum
 {
 	DOUBLE_CLICK_TEST_OFF,
 	DOUBLE_CLICK_TEST_MAYBE,
-	DOUBLE_CLICK_TEST_ON,
+	DOUBLE_CLICK_TEST_ON
 };
 
 enum
@@ -792,7 +792,7 @@ dialog_response_cb (GtkDialog *dialog, gint response_id, GConfChangeSet *changes
 {
 	if (response_id == GTK_RESPONSE_HELP)
 		capplet_help (GTK_WINDOW (dialog),
-			      "wgoscustdesk.xml",
+			      "user-guide.xml",
 			      "goscustperiph-5");
 	else
 		gtk_main_quit ();
