@@ -9,9 +9,56 @@
 static GQuark error_quark;
 enum
 {
-  INVALID_THEME_NAME,
+  INVALID_THEME_NAME
 };
 
+/* taken from gnome-desktop-item.c */
+static char *
+escape_string_and_dup (const char *s)
+{
+  char *return_value, *p;
+  const char *q;
+  int len = 0;
+
+  if (s == NULL)
+    return g_strdup("");
+	
+  q = s;
+  while (*q)
+    {
+      len++;
+      if (strchr ("\n\r\t\\", *q) != NULL)
+	len++;
+      q++;
+    }
+  return_value = p = (char *) g_malloc (len + 1);
+  do
+    {
+      switch (*s)
+	{
+	case '\t':
+	  *p++ = '\\';
+	  *p++ = 't';
+	  break;
+	case '\n':
+	  *p++ = '\\';
+	  *p++ = 'n';
+	  break;
+	case '\r':
+	  *p++ = '\\';
+	  *p++ = 'r';
+	  break;
+	case '\\':
+	  *p++ = '\\';
+	  *p++ = '\\';
+	  break;
+	default:
+	  *p++ = *s;
+	}
+    }
+  while (*s++);
+  return return_value;
+}
 
 static gboolean
 check_theme_name (const gchar  *theme_name,
@@ -55,6 +102,7 @@ setup_directory_structure (const gchar  *theme_name,
 static gboolean
 write_theme_to_disk (GnomeThemeMetaInfo  *meta_theme_info,
 		     const gchar         *theme_name,
+		     const gchar         *theme_description,
 		     GError             **error)
 {
   gchar *dir;
@@ -74,7 +122,7 @@ write_theme_to_disk (GnomeThemeMetaInfo  *meta_theme_info,
   gnome_vfs_truncate_handle (handle, 0);
 
   /* start making the theme file */
-  str = g_strdup_printf (theme_header, theme_name, theme_name);
+  str = g_strdup_printf (theme_header, theme_name, theme_description);
   gnome_vfs_write (handle, str, strlen (str), &bytes_written);
   g_free (str);
 
@@ -103,6 +151,7 @@ write_theme_to_disk (GnomeThemeMetaInfo  *meta_theme_info,
 static gboolean
 save_theme_to_disk (GnomeThemeMetaInfo  *meta_theme_info,
 		    const gchar         *theme_name,
+		    const gchar         *theme_description,
 		    GError             **error)
 {
   if (! check_theme_name (theme_name, error))
@@ -111,7 +160,7 @@ save_theme_to_disk (GnomeThemeMetaInfo  *meta_theme_info,
   if (! setup_directory_structure (theme_name, error))
     return FALSE;
 
-  if (! write_theme_to_disk (meta_theme_info, theme_name, error))
+  if (! write_theme_to_disk (meta_theme_info, theme_name, theme_description, error))
     return FALSE;
   
   return TRUE;
@@ -123,20 +172,33 @@ save_dialog_response (GtkWidget *save_dialog,
 		      gpointer   data)
 {
   GnomeThemeMetaInfo *meta_theme_info;
+  char *theme_description = NULL;
+  char *theme_name = NULL;
   GError *error = NULL;
 
   if (response_id == GTK_RESPONSE_OK)
     {
       GladeXML *dialog;
       GtkWidget *entry;
-      const char *theme_name;
-
+      GtkWidget *text_view;
+      GtkTextBuffer *buffer;
+      GtkTextIter start_iter;
+      GtkTextIter end_iter;
+      gchar *buffer_text;
+      
       dialog = gnome_theme_manager_get_theme_dialog ();
       entry = WID ("save_dialog_entry");
-
-      theme_name = gtk_entry_get_text (GTK_ENTRY (entry));
+      theme_name = escape_string_and_dup (gtk_entry_get_text (GTK_ENTRY (entry)));
+      
+      text_view = WID ("save_dialog_textview");
+      buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
+      gtk_text_buffer_get_start_iter (buffer, &start_iter);
+      gtk_text_buffer_get_end_iter (buffer, &end_iter);
+      buffer_text = gtk_text_buffer_get_text (buffer, &start_iter, &end_iter, FALSE);
+      theme_description = escape_string_and_dup (buffer_text);
+      g_free (buffer_text);
       meta_theme_info = (GnomeThemeMetaInfo *) g_object_get_data (G_OBJECT (save_dialog), "meta-theme-info");
-      if (! save_theme_to_disk (meta_theme_info, theme_name, &error))
+      if (! save_theme_to_disk (meta_theme_info, theme_name, theme_description, &error))
 	{
 	  goto out;
 	}
@@ -145,6 +207,8 @@ save_dialog_response (GtkWidget *save_dialog,
  out:
   g_clear_error (&error);
   gtk_widget_hide (save_dialog);
+  g_free (theme_name);
+  g_free (theme_description);
 }
 
 static inline gboolean
