@@ -27,6 +27,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gconf/gconf-client.h>
+#include <libgnome/gnome-help.h>
 
 #include "gnome-settings-accessibility-keyboard.h"
 #include "gnome-settings-daemon.h"
@@ -322,7 +323,7 @@ ax_response_callback (gint response_id, guint revert_controls_mask, gboolean ena
 									      0,
 									      GTK_MESSAGE_ERROR,
 									      GTK_BUTTONS_CLOSE,
-									      _("There was an error displaying help:"), 
+									      _("There was an error displaying help: %s"), 
 									      err->message);
 			    g_signal_connect (G_OBJECT (error_dialog),
 					      "response",
@@ -403,14 +404,14 @@ static void
 ax_slowkeys_warning_dialog_post (gboolean enabled)
 {
 	slowkeys_shortcut_val = enabled;
-	ax_stickykeys_dialog = warning_dialog_post (ax_slowkeys_dialog,
-						    &ax_slowkeys_dialog_hbox,
-						    (GCallback) ax_slowkeys_response,
-						    &slowkeys_shortcut_val,
-						    _("Slow Keys Alert"),
-						    _("You just held down the Shift key for 8 seconds.  This is the shortcut for the Slow Keys feature, which affects the way your keyboard works."),
-						    enabled ? _("Do you want to activate Slow Keys?") : 
-						    _("Do you want to deactivate Sticky Keys ?"));
+	ax_slowkeys_dialog = warning_dialog_post (ax_slowkeys_dialog,
+						  &ax_slowkeys_dialog_hbox,
+						  (GCallback) ax_slowkeys_response,
+						  &slowkeys_shortcut_val,
+						  _("Slow Keys Alert"),
+						  _("You just held down the Shift key for 8 seconds.  This is the shortcut for the Slow Keys feature, which affects the way your keyboard works."),
+						  enabled ? _("Do you want to activate Slow Keys?") : 
+						  _("Do you want to deactivate Slow Keys?"));
 }
 
 static void
@@ -425,7 +426,7 @@ ax_stickykeys_warning_dialog_post (gboolean enabled)
 						    enabled ? _("You just pressed the Shift key 5 times in a row.  This is the shortcut for the Sticky Keys feature, which affects the way your keyboard works.") : 
 						    _("You just pressed two keys at once, or pressed the Shift key 5 times in a row.  This turns off the Sticky Keys feature, which affects the way your keyboard works."),
 						    enabled ? _("Do you want to activate Sticky Keys?") : 
-						    _("Do you want to deactivate Sticky Keys ?"));
+						    _("Do you want to deactivate Sticky Keys?"));
 }
 
 static void
