@@ -221,13 +221,13 @@ typedef struct _ThemeHandle ThemeHandle;
 struct _ThemeHandle {
     GnomeVFSMethodHandle *handle;
 
-    // if we're listing themes....
+    /* if we're listing themes.... */
     GList *theme;
     GnomeVFSFileInfoOptions options;
     gboolean seen_dotdirectory;
     
-    // if we're doing a file....
-    gchar *uri; // the real URI
+    /* if we're doing a file.... */
+    gchar *uri; /* the real URI */
 };
 
 /* -- VFS method -- */
@@ -664,25 +664,39 @@ do_monitor_cancel(GnomeVFSMethod *method,
 static GnomeVFSMethod method = {
     sizeof(GnomeVFSMethod),
 
-    .open     = do_open,
-    .create   = do_create,
-    .close    = do_close,
-    .read     = do_read,
-    .write    = do_write,
-    .seek     = do_seek,
-    .tell     = do_tell,
-    .get_file_info_from_handle = do_get_file_info_from_handle,
+    do_open,
+    do_create,
+    do_close,
+    do_read,
+    do_write,
+    do_seek,
+    do_tell,
 
-    .open_directory  = do_open_directory,
-    .close_directory = do_close_directory,
-    .read_directory  = do_read_directory,
+    NULL,
 
-    .get_file_info = do_get_file_info,
-    .is_local      = do_is_local,
-    .unlink        = do_unlink,
+    do_open_directory,
+    do_close_directory,
+    do_read_directory,
+    do_get_file_info,
+    do_get_file_info_from_handle,
+    do_is_local,
 
-    .monitor_add = do_monitor_add,
-    .monitor_cancel = do_monitor_cancel
+    NULL,
+    NULL,
+    NULL,
+
+    do_unlink,
+
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+
+    do_monitor_add,
+    do_monitor_cancel,
+
+    NULL
 };
 
 GnomeVFSMethod*
